from django.contrib import admin
from django.contrib.admin.utils import unquote
from django.core.exceptions import PermissionDenied
from django.core.paginator import Paginator
from django.template.loader import render_to_string
from django.template.response import TemplateResponse
from django.urls import reverse
from django.utils.encoding import force_text
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _

from audit_trail.models import ContentType, Entity, Xaction
from audit_trail.queries import get_audit_qs


class AuditTrailAdmin(admin.ModelAdmin):

    def audit_col(self, obj):
        opts = self.model._meta
        app_label = opts.app_label
        model_name = opts.model_name
        history_url = reverse('admin:' + app_label + '_' + model_name + '_history',
                              args=(obj.id,))
        return render_to_string('audit_trail/admin_cl_icon.html',
                                {'history_url': history_url})

    audit_col.allow_tags = True
    audit_col.short_description = _('History')

    def history_view(self, request, object_id, extra_context=None):
        model = self.model
        obj = self.get_object(request, unquote(object_id))

        if obj is None:
            return self._get_obj_does_not_exist_redirect(request, model._meta, object_id)

        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        opts = model._meta
        app_label = opts.app_label
        action_list = get_audit_qs(model._meta.model_name, object_id)
        page = request.GET.get('page', 1)
        page_obj = Paginator(action_list, self.list_per_page).page(page)
        page_obj.object_list = page_obj.object_list. \
            values('transacted_on', 'transacted_by',
                   'transaction_type', 'field_name',
                   'current_val', 'current_val_type',
                   'previous_val', 'previous_val_type',
                   'xaction__display_text'
                   )

        context = dict(
            self.admin_site.each_context(request),
            title=_('Change History: %s' % force_text(obj)),
            action_list=page_obj,
            module_name=capfirst(force_text(opts.verbose_name_plural)),
            object=obj,
            opts=opts,
            preserved_filters=self.get_preserved_filters(request),
        )
        context.update(extra_context or {})

        request.current_app = self.admin_site.name

        return TemplateResponse(request, 'audit_trail/history.html', context)


@admin.register(Xaction)
class XactionAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'entity', 'xaction_type',
        'display_text', 'pfield_val',
        'ts', 'user',
    )
    list_filter = ('xaction_type', 'entity', 'ts')
    search_fields = ('entity__entity_name', 'display_text', 'user__email')


@admin.register(ContentType)
class ContentTypeAdmin(admin.ModelAdmin):
    search_fields = ('type',)


@admin.register(Entity)
class EntityAdmin(admin.ModelAdmin):
    list_display = ('entity_name', 'primary_field', 'display_format', 'status')
    search_fields = ('entity_name',)



from django.apps import apps
from django.contrib.auth import get_user_model
from django.db.models import F, Q


def get_audit_qs(model_name, object_id):
    FieldDiff = apps.get_model('audit_trail', 'FieldDiff')
    return FieldDiff.objects.filter(
        (Q(xaction__entity__entity_name=model_name, xaction__pfield_val=object_id)
         | Q(
                    xaction__xaction_parent_entity_child_fields__parent_entity_child_fields__parent_entity__entity_name=model_name,
                    xaction__xaction_parent_entity_child_fields__field_val=object_id))). \
        order_by('-xaction__ts'). \
        annotate(transacted_on=F('xaction__ts'),
                 transacted_by=F('xaction__user__' + get_user_model().USERNAME_FIELD),
                 transaction_type=F('xaction__xaction_type'),
                 field_name=F('field_id__name'),
                 current_val=F('curr_val__data'),
                 current_val_type=F('curr_val__content_type__type'),
                 previous_val=F('prev_val__data'),
                 previous_val_type=F('prev_val__content_type__type')
                 )


def get_audit_trail(model_name, object_id):
    return get_audit_qs(
        model_name, object_id
    ).values(
        'transacted_on', 'transacted_by',
        'transaction_type', 'field_name',
        'current_val', 'current_val_type',
        'previous_val', 'previous_val_type',
        'xaction__display_text'
    )



'''
Created on 02-Fab-2017

@author: dharmikp
'''

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from audit_trail.choices import xaction_type_choice


class Xaction(models.Model):
    """
    Transaction table for maintaining transaction of audit trail.
    """

    id = models.BigAutoField(primary_key=True)
    """
    Id field of xaction, it is auto increment.
    """

    entity = models.ForeignKey(
        'Entity',
        verbose_name='Entity',
        related_name='xactions',
        on_delete=models.CASCADE
    )
    """
    Foreign key to entity.
    """

    xaction_type = models.CharField(_('Xaction Type'), max_length=1,
                                    choices=xaction_type_choice, null=False,
                                    blank=False)
    """
    Type of the transaction.
    """

    display_text = models.CharField(_('Display Text'), max_length=128, null=True,
                                    blank=True)
    """
    Display text for the transaction.
    """

    pfield_val = models.CharField(_('Primary Field Value'), max_length=128,
                                  null=False, blank=False)
    """
    Primary field value of the transaction object.
    """

    ts = models.DateTimeField(_('Time Stamp'), null=False, blank=False,
                              auto_now_add=True)
    """
    Time stamp when this transaction is happen.
    """

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name='User',
        on_delete=models.CASCADE
    )
    """
    Foreign key to the user model.
    """

    class Meta:
        db_table = 'xaction'
        verbose_name_plural = _('Xactions')
        verbose_name = _('Xaction')
        ordering = ['-ts']

    def __str__(self):
        """Returns friendly description."""
        msg = '{xaction_type} {entity} {key} at {ts} by {user}'.format(
            xaction_type = self.xaction_type,
            entity=self.display_text,
            key=self.pfield_val,
            ts=self.ts,
            user=self.user
        )
        return msg



# Generated by Django 3.0.6 on 2020-05-31 17:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('audit_trail', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='entity',
            name='entity_name',
            field=models.CharField(max_length=64, unique=True, verbose_name='Entity Name'),
        ),
    ]
